# Generated by Django 1.11.5 on 2017-09-22 13:23
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Country",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("population", models.PositiveIntegerField(verbose_name="population")),
                ("tz", models.CharField(max_length=50)),
                ("visits", models.PositiveIntegerField()),
                ("commonwealth", models.NullBooleanField()),
                ("flag", models.FileField(upload_to="country/flags/")),
            ],
            options={"verbose_name_plural": "countries"},
        ),
        migrations.CreateModel(
            name="Person",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="full name")),
                ("friendly", models.BooleanField(default=True)),
                (
                    "country",
                    models.ForeignKey(
                        null=True, on_delete=django.db.models.deletion.CASCADE, to="app.Country"
                    ),
                ),
            ],
            options={"verbose_name_plural": "people"},
        ),
    ]
