# Generated by Django 2.0.1 on 2018-04-16 10:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("app", "0002_auto_20180416_0959")]

    operations = [
        migrations.AlterField(
            model_name="country",
            name="flag",
            field=models.FileField(blank=True, upload_to="country/flags/"),
        ),
        migrations.AlterField(
            model_name="country", name="tz", field=models.CharField(blank=True, max_length=50)
        ),
    ]
